
/*********************************************************************************

Example Usage


#include "bezier.inc"
union {
  sweep(0.25,0.001,.585) // radius detail dist-between-pnts
  texture { pigment {rgb <1,0,1> } finish { ambient 0.3 diffuse .5 } }
}
union {
  sweep_rc(0.25,0.001,.585) // radius detail dist-between-pnts
}
bezier circle handle length = R * $kappa
(4 * (sqrt(2) - 1) / 3) = 0.552284749830793
P(u) = A * (1 - u) + A' * 3u(1 - u) + B' * 3u(1 - u) + B * u

  bezier_subdivision2d()
    bezier_subdivision() will still divide points closer at sharper part of the curve
         bezier_length()
   bezier_point_spread() spreads points evenly along multiple segments
bezier_point_spread_rc() random colored spheres
*********************************************************************************/
#declare kappa = (sqrt(2) - 1) * 4/3;    //0.552284749830793

#declare S1 = seed (2341);
#declare S2 = seed (5521);
#declare S3 = seed (134515);

#macro bezier_subdivision2d(p1,h1,h2,p2,Div) // Div range 0-1
 #local pnt = array[4][4] { { p1,h1,h2,p2 } { <0,0>,<0,0>,<0,0>,<0,0> }
   {<0,0>,<0,0>,<0,0>,<0,0>}{<0,0>,<0,0>,<0,0>,<0,0>} };
 #local j=1; #local i=0;
 #while (j<4)
     #local i=0;
     #while (i < (4-j))
        #local pnt[j][i]= pnt[j-1][i]*(1-Div) + pnt[j-1][i+1]*Div;
        #local i=i+1;
     #end
     #local j=j+1;
 #end
 pnt[3][0]
 // return divide point
 
 // 2 new domains
 // curve___1 = pnt[0][0] pnt[1][0] pnt[2][0] pnt[3][0]
 // remainder = pnt[3][0] pnt[2][1] pnt[1][2] pnt[0][3]
#end

#macro bezier_subdivision(P1,H1,H2,P2,Div) // Div range 0-1
 #local Pnt = array[4][4] { { P1,H1,H2,P2 } { <0,0,0>,<0,0,0>,<0,0,0>,<0,0,0> }
   {<0,0,0>,<0,0,0>,<0,0,0>,<0,0,0>}{<0,0,0>,<0,0,0>,<0,0,0>,<0,0,0>} };
 #local j=1; #local i=0;
 #while (j<4)
     #local i=0;
     #while (i < (4-j))
        #local Pnt[j][i] = Pnt[j-1][i]*(1-Div) + Pnt[j-1][i+1]*Div;
        #local i=i+1;
     #end
     #local j=j+1;
 #end
 Pnt[3][0]
 // return divide point
 
 // 2 new domains
 // curve___1 = pnt[0][0] pnt[1][0] pnt[2][0] pnt[3][0]
 // remainder = pnt[3][0] pnt[2][1] pnt[1][2] pnt[0][3]
#end

#macro bezier_curve(p1,h1,h2,p2,detail) // <x,y> or <x,y,0,> or <x,0,y>
           //(node, ctrlh, ctrlh, node, .01)
     #local C = 3*(h1-p1);
     #local B = 3*(h2-h1)-C;
     #local A = p2-p1-C-B;
     
     #local i = 0; 
     #while (i<=1)
         #local xy= A*pow(i,3)+B*pow(i,2)+C*i+p1;
         #local i=i+detail;
     #end
     #local i=1;
     #local xy = A*pow(i,3)+B*pow(i,2)+C*i+p1;
     echov2(xy) echo("\n")
     #local xy=p2;
     echov2(xy) echo("\n") // same numbers
#end

#macro bezier_length(SplineArray)
     #local i=0;  #local n=0;
     #while (i < dimension_size(SplineArray, 1)/4)   
     #local n=n+bezier_len(
        SplineArray[i*4]+<0,0,0>,
        SplineArray[i*4+1]+<0,0,0>,
        SplineArray[i*4+2]+<0,0,0>,
        SplineArray[i*4+3]+<0,0,0>);
       #local i=i+1;
     #end
    n
#end

#macro bezier_len(P1,H1,H2,P2)
     #local C = 3*(H1-P1);
     #local B = 3*(H2-H1)-C;
     #local A = P2-P1-C-B;
     #local prv = P1; #local n = 0;
     #local i = 0; 
     #while (i<=1)
         #local _v = A*pow(i,3)+B*pow(i,2)+C*i+P1;
         #local n = n+vlength(_v-prv);
         #local prv = _v;
         #local i=i+.01;
     #end
     n
#end

#macro bezier_point_spread(P1,H1,H2,P2,detail,dist,Radius)
             //(<x,y,0>3D input<x,0,y>, 0.001,   1,      )
     #local C = 3*(H1-P1);
     #local B = 3*(H2-H1)-C;
     #local A = P2-P1-C-B;
     
     #local i = 0; #local XYZ=P1;
     // do something with point1 (P1) --- start out of loop2 range, from start macro
     
     #while (i<1)
         #while ((i<1) & (vlength(XYZ-Prev)<dist))
             #local XYZ= A*pow(i,3)+B*pow(i,2)+C*i+P1;
             #local i=i+detail;
         #end
         #if (vlength(XYZ-Prev)>=dist)
           #if ((0) & (i) & (vlength(XYZ-Prev) < dist * 2))
                  cylinder { Prev+(Prev-XYZ)*.001,XYZ,Radius }
           #else sphere { XYZ,Radius } #end
           #declare J=J+1;
           #declare Prev=XYZ;
         #end
     #end
#end
#macro bezier_point_spread_rc(P1,H1,H2,P2,detail,dist,Radius)
             //(<x,y,0>3D input<x,0,y>, 0.001,      1,      )
     #local C = 3*(H1-P1);
     #local B = 3*(H2-H1)-C;
     #local A = P2-P1-C-B;
     
     #local i = 0; #local XYZ=P1;
     // do something with point1 (P1) --- start out of loop2 range, from start macro
     
     #while (i<1)
         #while ((i<1) & (vlength(XYZ-Prev)<dist))
             #local XYZ= A*pow(i,3)+B*pow(i,2)+C*i+P1;
             #local i=i+detail;
         #end
         #if (vlength(XYZ-Prev)>=dist)
           #if ((0) & (i) & (vlength(XYZ-Prev) < dist * 2))
                 cylinder { Prev+(Prev-XYZ)*.001,XYZ,Radius
           #else sphere { XYZ,Radius
           #end
           texture { pigment {rgb <rand(S3),rand(S2),rand(S1)> } finish { ambient 0.3 diffuse .5 } }
           }
           #declare J=J+1;
           #declare Prev=XYZ;
         #end
     #end
#end


#declare pov_new_p = prism {
linear_sweep
bezier_spline
0.0, //top
-1.0, //bottom
144,
<-9.2,-19.525406>,<-9.2,-19.440014>,<-8.838889,-18.091264>,<-8.397532,-16.528183>,
<-8.397532,-16.528183>,<-7.956175,-14.965102>,<-7.368484,-12.876218>,<-7.091552,-11.886218>,
<-7.091552,-11.886218>,<-6.814619,-10.896218>,<-6.377995,-9.347766>,<-6.121276,-8.445214>,
<-6.121276,-8.445214>,<-6.093917,-8.349029>,<-5.681872,-6.900396>,<-5.654513,-6.804211>,
<-5.654513,-6.804211>,<-5.733833,-6.743314>,<-5.912544,-6.606111>,<-5.991864,-6.545214>,
<-5.991864,-6.545214>,<-6.949847,-5.809737>,<-8.173005,-4.57589>,<-8.058069,-4.460954>,
<-8.058069,-4.460954>,<-8.033586,-4.436471>,<-7.729255,-4.618013>,<-7.381778,-4.86438>,
<-7.381778,-4.86438>,<-6.683262,-5.359639>,<-5.788497,-5.836218>,<-5.557179,-5.836218>,
<-5.557179,-5.836218>,<-5.466318,-5.836218>,<-5.361915,-5.66957>,<-5.290922,-5.411218>,
<-5.290922,-5.411218>,<-5.226689,-5.177468>,<-5.052159,-4.536218>,<-4.903076,-3.986218>,
<-4.903076,-3.986218>,<-4.73495,-3.365963>,<-4.57847,-2.976851>,<-4.491008,-2.961551>,
<-4.491008,-2.961551>,<-4.413454,-2.947984>,<-4.22395,-3.115874>,<-4.069889,-3.334639>,
<-4.069889,-3.334639>,<-3.714304,-3.839567>,<-2.760591,-4.576203>,<-2.063077,-4.884677>,
<-2.063077,-4.884677>,<-1.362151,-5.194659>,<-0.372358,-5.370874>,<0.225,-5.292026>,
<0.225,-5.292026>,<0.793263,-5.217019>,<0.813394,-5.28537>,<0.4,-5.886218>,
<0.4,-5.886218>,<0.235,-6.126037>,<0.1,-6.370395>,<0.1,-6.429236>,
<0.1,-6.429236>,<0.1,-6.59938>,<0.511971,-6.555244>,<1.443472,-6.285306>,
<1.443472,-6.285306>,<4.012626,-5.540794>,<5.89133,-3.893033>,<6.615852,-1.74875>,
<6.615852,-1.74875>,<7.048219,-0.469122>,<6.852897,1.205381>,<6.124482,2.463782>,
<6.124482,2.463782>,<5.775273,3.06707>,<4.88527,3.991084>,<4.15,4.513718>,
<4.15,4.513718>,<0.932457,6.800763>,<-3.878082,6.807377>,<-7.198783,4.52932>,
<-7.198783,4.52932>,<-7.445614,4.35999>,<-7.929364,3.935457>,<-8.273783,3.585914>,
<-8.273783,3.585914>,<-8.97885,2.870357>,<-9.068581,2.945591>,<-8.545709,3.813906>,
<-8.545709,3.813906>,<-7.705588,5.209065>,<-6.418528,6.351022>,<-4.770785,7.163245>,
<-4.770785,7.163245>,<-2.482826,8.29105>,<0.356032,8.547967>,<2.944599,7.861487>,
<2.944599,7.861487>,<5.740213,7.120099>,<8.392345,4.800938>,<9.190649,2.39962>,
<9.190649,2.39962>,<9.638526,1.052392>,<9.701838,-0.633887>,<9.353438,-1.936218>,
<9.353438,-1.936218>,<8.9653,-3.387091>,<7.903633,-4.934387>,<6.547978,-6.024955>,
<6.547978,-6.024955>,<4.684337,-7.524178>,<3.017211,-8.113809>,<0.378661,-8.206925>,
<0.378661,-8.206925>,<-0.896005,-8.251909>,<-1.21953,-8.29295>,<-1.334904,-8.424298>,
<-1.334904,-8.424298>,<-1.46537,-8.572831>,<-4.591501,-13.099385>,<-7.59325,-17.486218>,
<-7.59325,-17.486218>,<-8.924857,-19.432264>,<-9.2,-19.781458>,<-9.2,-19.525406>,
<0,-3.7704>,<-2.082334,-3.7704>,<-3.7704,-2.082334>,<-3.7704,0>,
<-3.7704,0>,<-3.7704,2.082334>,<-2.082334,3.7704>,<0,3.7704>,
<0,3.7704>,<2.082334,3.7704>,<3.7704,2.082334>,<3.7704,0>,
<3.7704,0>,<3.7704,-2.082334>,<2.082334,-3.7704>,<0,-3.7704>      
}
#declare pov_P_prism=pov_new_p;
#declare P_bez = array[144] {
<-9.2,-19.525406>,<-9.2,-19.440014>,<-8.838889,-18.091264>,<-8.397532,-16.528183>,
<-8.397532,-16.528183>,<-7.956175,-14.965102>,<-7.368484,-12.876218>,<-7.091552,-11.886218>,
<-7.091552,-11.886218>,<-6.814619,-10.896218>,<-6.377995,-9.347766>,<-6.121276,-8.445214>,
<-6.121276,-8.445214>,<-6.093917,-8.349029>,<-5.681872,-6.900396>,<-5.654513,-6.804211>,
<-5.654513,-6.804211>,<-5.733833,-6.743314>,<-5.912544,-6.606111>,<-5.991864,-6.545214>,
<-5.991864,-6.545214>,<-6.949847,-5.809737>,<-8.173005,-4.57589>,<-8.058069,-4.460954>,
<-8.058069,-4.460954>,<-8.033586,-4.436471>,<-7.729255,-4.618013>,<-7.381778,-4.86438>,
<-7.381778,-4.86438>,<-6.683262,-5.359639>,<-5.788497,-5.836218>,<-5.557179,-5.836218>,
<-5.557179,-5.836218>,<-5.466318,-5.836218>,<-5.361915,-5.66957>,<-5.290922,-5.411218>,
<-5.290922,-5.411218>,<-5.226689,-5.177468>,<-5.052159,-4.536218>,<-4.903076,-3.986218>,
<-4.903076,-3.986218>,<-4.73495,-3.365963>,<-4.57847,-2.976851>,<-4.491008,-2.961551>,
<-4.491008,-2.961551>,<-4.413454,-2.947984>,<-4.22395,-3.115874>,<-4.069889,-3.334639>,
<-4.069889,-3.334639>,<-3.714304,-3.839567>,<-2.760591,-4.576203>,<-2.063077,-4.884677>,
<-2.063077,-4.884677>,<-1.362151,-5.194659>,<-0.372358,-5.370874>,<0.225,-5.292026>,
<0.225,-5.292026>,<0.793263,-5.217019>,<0.813394,-5.28537>,<0.4,-5.886218>,
<0.4,-5.886218>,<0.235,-6.126037>,<0.1,-6.370395>,<0.1,-6.429236>,
<0.1,-6.429236>,<0.1,-6.59938>,<0.511971,-6.555244>,<1.443472,-6.285306>,
<1.443472,-6.285306>,<4.012626,-5.540794>,<5.89133,-3.893033>,<6.615852,-1.74875>,
<6.615852,-1.74875>,<7.048219,-0.469122>,<6.852897,1.205381>,<6.124482,2.463782>,
<6.124482,2.463782>,<5.775273,3.06707>,<4.88527,3.991084>,<4.15,4.513718>,
<4.15,4.513718>,<0.932457,6.800763>,<-3.878082,6.807377>,<-7.198783,4.52932>,
<-7.198783,4.52932>,<-7.445614,4.35999>,<-7.929364,3.935457>,<-8.273783,3.585914>,
<-8.273783,3.585914>,<-8.97885,2.870357>,<-9.068581,2.945591>,<-8.545709,3.813906>,
<-8.545709,3.813906>,<-7.705588,5.209065>,<-6.418528,6.351022>,<-4.770785,7.163245>,
<-4.770785,7.163245>,<-2.482826,8.29105>,<0.356032,8.547967>,<2.944599,7.861487>,
<2.944599,7.861487>,<5.740213,7.120099>,<8.392345,4.800938>,<9.190649,2.39962>,
<9.190649,2.39962>,<9.638526,1.052392>,<9.701838,-0.633887>,<9.353438,-1.936218>,
<9.353438,-1.936218>,<8.9653,-3.387091>,<7.903633,-4.934387>,<6.547978,-6.024955>,
<6.547978,-6.024955>,<4.684337,-7.524178>,<3.017211,-8.113809>,<0.378661,-8.206925>,
<0.378661,-8.206925>,<-0.896005,-8.251909>,<-1.21953,-8.29295>,<-1.334904,-8.424298>,
<-1.334904,-8.424298>,<-1.46537,-8.572831>,<-4.591501,-13.099385>,<-7.59325,-17.486218>,
<-7.59325,-17.486218>,<-8.924857,-19.432264>,<-9.2,-19.781458>,<-9.2,-19.525406>,
<0,-3.7704>,<-2.082334,-3.7704>,<-3.7704,-2.082334>,<-3.7704,0>,
<-3.7704,0>,<-3.7704,2.082334>,<-2.082334,3.7704>,<0,3.7704>,
<0,3.7704>,<2.082334,3.7704>,<3.7704,2.082334>,<3.7704,0>,
<3.7704,0>,<3.7704,-2.082334>,<2.082334,-3.7704>,<0,-3.7704>
}
#declare P_bez_noc = array[144-16] {
<-9.2,-19.525406>,<-9.2,-19.440014>,<-8.838889,-18.091264>,<-8.397532,-16.528183>,
<-8.397532,-16.528183>,<-7.956175,-14.965102>,<-7.368484,-12.876218>,<-7.091552,-11.886218>,
<-7.091552,-11.886218>,<-6.814619,-10.896218>,<-6.377995,-9.347766>,<-6.121276,-8.445214>,
<-6.121276,-8.445214>,<-6.093917,-8.349029>,<-5.681872,-6.900396>,<-5.654513,-6.804211>,
<-5.654513,-6.804211>,<-5.733833,-6.743314>,<-5.912544,-6.606111>,<-5.991864,-6.545214>,
<-5.991864,-6.545214>,<-6.949847,-5.809737>,<-8.173005,-4.57589>,<-8.058069,-4.460954>,
<-8.058069,-4.460954>,<-8.033586,-4.436471>,<-7.729255,-4.618013>,<-7.381778,-4.86438>,
<-7.381778,-4.86438>,<-6.683262,-5.359639>,<-5.788497,-5.836218>,<-5.557179,-5.836218>,
<-5.557179,-5.836218>,<-5.466318,-5.836218>,<-5.361915,-5.66957>,<-5.290922,-5.411218>,
<-5.290922,-5.411218>,<-5.226689,-5.177468>,<-5.052159,-4.536218>,<-4.903076,-3.986218>,
<-4.903076,-3.986218>,<-4.73495,-3.365963>,<-4.57847,-2.976851>,<-4.491008,-2.961551>,
<-4.491008,-2.961551>,<-4.413454,-2.947984>,<-4.22395,-3.115874>,<-4.069889,-3.334639>,
<-4.069889,-3.334639>,<-3.714304,-3.839567>,<-2.760591,-4.576203>,<-2.063077,-4.884677>,
<-2.063077,-4.884677>,<-1.362151,-5.194659>,<-0.372358,-5.370874>,<0.225,-5.292026>,
<0.225,-5.292026>,<0.793263,-5.217019>,<0.813394,-5.28537>,<0.4,-5.886218>,
<0.4,-5.886218>,<0.235,-6.126037>,<0.1,-6.370395>,<0.1,-6.429236>,
<0.1,-6.429236>,<0.1,-6.59938>,<0.511971,-6.555244>,<1.443472,-6.285306>,
<1.443472,-6.285306>,<4.012626,-5.540794>,<5.89133,-3.893033>,<6.615852,-1.74875>,
<6.615852,-1.74875>,<7.048219,-0.469122>,<6.852897,1.205381>,<6.124482,2.463782>,
<6.124482,2.463782>,<5.775273,3.06707>,<4.88527,3.991084>,<4.15,4.513718>,
<4.15,4.513718>,<0.932457,6.800763>,<-3.878082,6.807377>,<-7.198783,4.52932>,
<-7.198783,4.52932>,<-7.445614,4.35999>,<-7.929364,3.935457>,<-8.273783,3.585914>,
<-8.273783,3.585914>,<-8.97885,2.870357>,<-9.068581,2.945591>,<-8.545709,3.813906>,
<-8.545709,3.813906>,<-7.705588,5.209065>,<-6.418528,6.351022>,<-4.770785,7.163245>,
<-4.770785,7.163245>,<-2.482826,8.29105>,<0.356032,8.547967>,<2.944599,7.861487>,
<2.944599,7.861487>,<5.740213,7.120099>,<8.392345,4.800938>,<9.190649,2.39962>,
<9.190649,2.39962>,<9.638526,1.052392>,<9.701838,-0.633887>,<9.353438,-1.936218>,
<9.353438,-1.936218>,<8.9653,-3.387091>,<7.903633,-4.934387>,<6.547978,-6.024955>,
<6.547978,-6.024955>,<4.684337,-7.524178>,<3.017211,-8.113809>,<0.378661,-8.206925>,
<0.378661,-8.206925>,<-0.896005,-8.251909>,<-1.21953,-8.29295>,<-1.334904,-8.424298>,
<-1.334904,-8.424298>,<-1.46537,-8.572831>,<-4.591501,-13.099385>,<-7.59325,-17.486218>,
<-7.59325,-17.486218>,<-8.924857,-19.432264>,<-9.2,-19.781458>,<-9.2,-19.525406>,
}
//echov(bezier_subdivision(P_bez[0]+<0,0,0>,P_bez[1]+<0,0,0>,P_bez[2]+<0,0,0>,P_bez[3]+<0,0,0>,.8)) echo(" subdivision\n")
//echov2(bezier_subdivision2d(P_bez[0],P_bez[1],P_bez[2],P_bez[3],.8)) echo(" subdivision\n")
//echof(bezier_len(P_bez[0]+<0,0,0>,P_bez[1]+<0,0,0>,P_bez[2]+<0,0,0>,P_bez[3]+<0,0,0>)) echo(" Bez Length\n")
echo("\n")
#macro sweep(Radius,detail,dist)
    #declare J=0; // counter
    #declare Prev = P_bez[0]*10;
    // start out of range
    //bezier_point_spread(P1,H1,H2,P2,detail,dist,Radius)
    #local i=0;
    #while (i < 144/4)
      bezier_point_spread(
        P_bez[i*4]+<0,0,0>,
        P_bez[i*4+1]+<0,0,0>,
        P_bez[i*4+2]+<0,0,0>,
        P_bez[i*4+3]+<0,0,0>,
        detail,dist,Radius
      )   //detail min=.001 iter increment

      #local i=i+1;
    #end
    echo("Created ") echoi(J) echo(" points*****///////SWEEP///////*****\n")

#end
#macro sweep_rc(Radius,detail,dist)
    #declare J =0;
    #declare Prev = P_bez[0]*10;
    // start out of range
    //bezier_point_spread(P1,H1,H2,P2,detail,dist,Radius)
    #local i=0;
    #while (i < 144/4)
      bezier_point_spread_rc(
        P_bez[i*4]+<0,0,0>,
        P_bez[i*4+1]+<0,0,0>,
        P_bez[i*4+2]+<0,0,0>,
        P_bez[i*4+3]+<0,0,0>,
        detail,dist,Radius
      )

      #local i=i+1;
    #end
    echo("Created ") echoi(J) echo(" points*****///////SWEEP///////*****\n")
#end 

#macro sweep_pnts_array_yz(Radius,detail,dist,arr,sw) //(0.24,0.001,.585)    //.585=232 points    .587=230 pnts    .589=229 pnts

    #declare J=0;         // capital J = global counter
    #declare Prev = arr[0]*10;
    // start out of range

    #local i=0;
    #while (i < dimension_size(arr,1)/4)
      point_spread_count(
        <0,arr[i*4].y,arr[i*4].x>,
        <0,arr[i*4+1].y,arr[i*4+1].x>,
        <0,arr[i*4+2].y,arr[i*4+2].x>,
        <0,arr[i*4+3].y,arr[i*4+3].x>,
        detail,dist,Radius
      )

      #local i=i+1;
    #end
    
    #if (sw) #declare aSpline = array [J]
    #else #declare bSpline = array [J]
    #end
    
    #declare J=0;         // capital J = global counter
    #declare Prev = arr[0]*10;
    // start out of range

    #local i=0;
    #while (i < dimension_size(arr,1)/4)
      point_spread(
        <0,arr[i*4].y,arr[i*4].x>,
        <0,arr[i*4+1].y,arr[i*4+1].x>,
        <0,arr[i*4+2].y,arr[i*4+2].x>,
        <0,arr[i*4+3].y,arr[i*4+3].x>,
        detail,dist,Radius.sw
      )

      #local i=i+1;
    #end
    echo("Created ") echoi(J) echo(" points*****///////SWEEP///////*****\n")
    

#end
#macro sweep_pnts_array_xy(Radius,detail,dist,arr,sw) //(0.24,0.001,.585)    //.585=232 points    .587=230 pnts    .589=229 pnts

    #declare J=0;         // capital J = global counter
    #declare Prev = arr[0]*10;
    // start out of range

    #local i=0;
    #while (i < dimension_size(arr,1)/4)
      point_spread_count(
        arr[i*4]+<0,0,0>,
        arr[i*4+1]+<0,0,0>,
        arr[i*4+2]+<0,0,0>,
        arr[i*4+3]+<0,0,0>,
        detail,dist,Radius
      )

      #local i=i+1;
    #end
    
    #if (sw) #declare aSpline = array [J]
    #else #declare bSpline = array [J]
    #end
    
    #declare J=0;         // capital J = global counter
    #declare Prev = arr[0]*10;
    // start out of range

    #local i=0;
    #while (i < dimension_size(arr,1)/4)
      point_spread(
        arr[i*4]+<0,0,0>,
        arr[i*4+1]+<0,0,0>,
        arr[i*4+2]+<0,0,0>,
        arr[i*4+3]+<0,0,0>,
        detail,dist,Radius,sw
      )

      #local i=i+1;
    #end
    echo("Created ") echoi(J) echo(" points*****///////SWEEP///////*****\n")
    

#end



#macro setEdge(a,b)
     #declare Edge_Arr[NumEdges] = <a,b>;
     #declare NumEdges = NumEdges+1;
#end
#macro setFace(_v)
     #declare Face_Arr[NumFaces] = _v;
     #declare NumFaces = NumFaces+1;
#end  
#macro triangle2egdes(_v,_vv)
     #declare Edge_Arr[NumEdges] = <_v.x,_v.y>;
     #declare NumEdges = NumEdges+1; 
     #declare Edge_Arr[NumEdges] = <_v.z,_v.y>;
     #declare NumEdges = NumEdges+1; 
     #declare Edge_Arr[NumEdges] = <_v.x,_v.z>;
     #declare NumEdges = NumEdges+1; 

     #declare Edge_Arr[NumEdges] = <_vv.x,_vv.y>;
     #declare NumEdges = NumEdges+1; 
     #declare Edge_Arr[NumEdges] = <_vv.z,_vv.y>;
     #declare NumEdges = NumEdges+1; 
     #declare Edge_Arr[NumEdges] = <_vv.x,_vv.z>;
     #declare NumEdges = NumEdges+1;

#end

#macro bezierPrism(exH,rm1,rm2,m1,m2,slices,SubEdges,close)      // assumes clockwise perimeter, else u get anticlok faces
  // extrusionheight,radiusmod1,radiusmod2, plane mod = handle rotation deg w/h

#local bsize = dimension_size(bSpline, 1);
#local asize = dimension_size(aSpline, 1);
   #declare NumVertices = int((asize+bsize)*slices*1.35); // will be reset to actual, I dont Know

   #declare V_vec_Arr=array[NumVertices] 
   #declare N_vec_Arr=array[NumVertices] 
   #declare UV_vec_Arr=array[NumVertices]
   #declare NumEdges = 0;
   #declare Edge_Arr=array[NumVertices*3]
   #declare NumFaces = 0;
   #declare Face_Arr=array[NumVertices*2] // face indices, needs a rename
   cr() echoi(NumVertices) echo(" Allocated")
   

   #local i=0; #local n=bsize;
   
   #while (i<n)
      #declare N_vec_Arr[i] = y;
      #declare V_vec_Arr[i] = bSpline[i];
      #declare UV_vec_Arr[i] = <1,0>;
      #if (i-i) #declare Edge_Arr[NumEdges] = <i-1,i>;
              #declare NumEdges = NumEdges+1;
      #end
      #local i=i+1;
   #end
   #local n=n+asize;
   #while (i<n)
      #declare N_vec_Arr[i] = y;
      #declare UV_vec_Arr[i] = <1,0>;
      #declare V_vec_Arr[i] = aSpline[i-bsize]+<0,exH,0>;
      #if (i-i) #declare Edge_Arr[NumEdges] = <i-1,i>;
              #declare NumEdges = NumEdges+1;
      #end
      #local i=i+1;
   #end
   // close perimeter path  
   #if (0)
     #declare Edge_Arr[NumEdges] = <bsize-1,0>;
     #declare NumEdges = NumEdges+1;   
   #end
   #local kk = n+(slices-2);
   
   #local hr = y*exH*kappa;                     // reset
   #local hr = hr +  y*exH*(1-kappa)*rm2;
  
  #local Swtch = 0; #local aa = bsize; #local i = 0; #local j = 1; #local k = n;
  #local incdec = 1; #local trap = 1;  #local dtheta = 1 / (slices-1);

  
  #while (i < bsize) /////////////////////////////////// START \\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\
    #local j = 1; #local ipnt = V_vec_Arr[i];
    #local aapnt = V_vec_Arr[aa];             // backbone elevated
    #local wr = (ipnt+y*aapnt.y-aapnt)*kappa;             // perim-pnt <- backbone at y0=radius

    #local _v = vcross(y,wr);
    #local wr = wr+ (ipnt+y*aapnt.y-aapnt)*(1-kappa)*rm1; //radius modifier 1 
    
       // handle rot plane modifiers
       #local wr = rotation(radians(1*m1),_v,wr);
       #local hr = y*aapnt.y*kappa;
       #local hr = hr +  y*aapnt.y*(1-kappa)*rm2;
       #local hr = rotation(radians(-1*m2),_v,hr);

       
       
    #local V=RangeMM(i,0,bsize,Vmin,Vmax);
    #while (j < slices-1)
       #local U=RangeMM(j,0,slices,Umin,Umax); // min max radians
       #declare UV_vec_Arr[k]=<U,V>;
                                               ///////H////////
       #declare V_vec_Arr[k] = bezier_pnt(ipnt,ipnt+hr,aapnt+wr,aapnt,j*dtheta);
       #declare N_vec_Arr[k] = vnormalize(V_vec_Arr[k]-aapnt); // set something, actual=Norms()
       #if (j=1)
           #declare Edge_Arr[NumEdges] = <i,k>;
           #declare NumEdges = NumEdges+1;
        #else
           #declare Edge_Arr[NumEdges] = <k-1,k>;
           #declare NumEdges = NumEdges+1;           
       #end
       #if (j=slices-2)  // last one
           #declare Edge_Arr[NumEdges] = <k,aa>;
           #declare NumEdges = NumEdges+1;
       #end
 
      #local k=k+1; #declare NumVertices = k;
      #local j=j+1;
    #end

    #if (i) 
      #if (Swtch)
        #local Swtch = 0;
        // incremented backbone last time   aa               
        #local j=1;
        #while (j < slices-2)
           #if (j=1)
              setFace(<aa-incdec,k-j-(slices-2),k-j>); //      _|
              setFace(<aa-incdec,k-j,aa>);      //             |/
           #end
           setFace(<k-j-(slices-2),k-j-(slices-2)-1,k-j-1>);  //      _|
           setFace(<k-j-(slices-2),k-j-1,k-j>); //                    |/
           //#end                               )
           #local j=j+1; 
        #end
        //setFace(<k-j-(slices-2)-1,i,k-j-1>); //             \/
        setFace(<k-j-(slices-2),i,k-j>); //             \/
        
        //#if (i = bsize-1) /// hmm  #end assuming 1st and last point connect same bakbone pnt1 ?!? fuzzy logic
      #else 
        #local j=1;
        setFace(<aa,k-j-(slices-2),k-j>);
        
        #while (j < slices-1)
           #if (j=slices-2)
              setFace(<k-j,k-j-(slices-2),i>);
              setFace(<k-j-(slices-2),i-1,i>);
           #else
              setFace(<k-j,k-j-(slices-2),k-j-1>);
              setFace(<k-j-(slices-2),k-j-(slices-2)-1,k-j-1>);
           #end
           #local j=j+1;
        #end      
      #end// swtch
      
      #if ((i = bsize-1) & (close)) //done. close circular shape
         #local j=1;
         setFace(<aa,k-j,kk-j>);
         #while (j < slices-1)
            #if (j=slices-2)
               setFace(<kk-j,k-j,0>);
               setFace(<k-j,i,0>);
            #else
               setFace(<kk-j,k-j,kk-j-1>);
               setFace(<k-j,k-j-1,kk-j-1>);
            #end
            
            #local j=j+1;
         #end
      #end
    #end // i
    #local i=i+1;      #if (1) // for pov logo only                                             // corner of 50
                          #if ((i>44) & (i<52)) #local trap = 0;   // fix for perimeter not following bakbone
                          #else #local trap = 1; #end
                       #end
                    //n
    #if (asize > 1)
    #if (aa >= n-1) #local incdec = -1; #end
    #if ((aa+incdec > bsize-1) & (i < bsize))     // stay in range
         #if ((vlength(V_vec_Arr[aa+incdec]-bSpline[i])<vlength(V_vec_Arr[aa]-bSpline[i])) & (trap))
             #local aa=aa+incdec;
             // if next backbone pnt is closer, use it
             #local Swtch = 1;
             #local i=i-1;  // dont goto next perimeter pnt
         #end
    #end
    #end
  #end
  
  echo("\nNum Verts ********* ")
  echoi(NumVertices)
  echo("\nNum Faces ********* ")
  echoi(NumFaces)
  echo("\nNum Edges ********* ")
  echoi(NumEdges) cr() cr()
#end

#macro bezierPrismQ(exH,rm1,rm2,m1,m2,slices,SubEdges,close)      // assumes clockwise perimeter, else u get anticlok faces
  // extrusionheight,radiusmod1,radiusmod2, plane mod = handle rotation deg w/h

#local bsize = dimension_size(bSpline, 1);
#local asize = dimension_size(aSpline, 1);
   #declare NumVertices = int((asize+bsize)*slices*1.35); // will be reset to actual, I dont Know

   #declare V_vec_Arr=array[NumVertices] 
   #declare N_vec_Arr=array[NumVertices] 
   #declare UV_vec_Arr=array[NumVertices]
   #declare NumEdges = 0;
   #declare Edge_Arr=array[NumVertices*3]
   #declare NumFaces = 0;
   #declare Face_Arr=array[NumVertices*2] // face indices, needs a rename
   cr() echoi(NumVertices) echo(" Allocated")
   

   #local i=0; #local n=bsize;
   
   #while (i<n)
      #declare N_vec_Arr[i] = y;
      #declare V_vec_Arr[i] = bSpline[i];
      #declare UV_vec_Arr[i] = <1,0>;
      #if (i-i) #declare Edge_Arr[NumEdges] = <i-1,i>;
              #declare NumEdges = NumEdges+1;
      #end
      #local i=i+1;
   #end
   #local n=n+asize;
   #while (i<n)
      #declare N_vec_Arr[i] = y;
      #declare UV_vec_Arr[i] = <1,0>;
      #declare V_vec_Arr[i] = aSpline[i-bsize]+<0,exH,0>;
      #if (i-i) #declare Edge_Arr[NumEdges] = <i-1,i>;
              #declare NumEdges = NumEdges+1;
      #end
      #local i=i+1;
   #end
   // close perimeter path  
   #if (0)
     #declare Edge_Arr[NumEdges] = <bsize-1,0>;
     #declare NumEdges = NumEdges+1;   
   #end
   #local kk = n+(slices-2);
   
   #local hr = y*exH*kappa;                     // reset
   #local hr = hr +  y*exH*(1-kappa)*rm2;
  
  #local Swtch = 0; #local aa = bsize; #local i = 0; #local j = 1; #local k = n;
  #local incdec = 1; #local trap = 1;  #local dtheta = 1 / (slices-1);

  
  #while (i < bsize) /////////////////////////////////// START \\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\
    #local j = 1; #local ipnt = V_vec_Arr[i];
    #local aapnt = V_vec_Arr[aa];             // backbone elevated
    #local wr = (ipnt+y*aapnt.y-aapnt)*kappa;             // perim-pnt <- backbone at y0=radius

    #local _v = vcross(y,wr);
    #local wr = wr+ (ipnt+y*aapnt.y-aapnt)*(1-kappa)*rm1; //radius modifier 1 
    
       // handle rot plane modifiers
       #local wr = rotation(radians(1*m1),_v,wr);
       #local hr = y*aapnt.y*kappa;
       #local hr = hr +  y*aapnt.y*(1-kappa)*rm2;
       #local hr = rotation(radians(-1*m2),_v,hr);

       
       
    #local V=RangeMM(i,0,bsize,Vmin,Vmax);
    #while (j < slices-1)
       #local U=RangeMM(j,0,slices,Umin,Umax); // min max radians
       #declare UV_vec_Arr[k]=<U,V>;
                                               ///////H////////
       #declare V_vec_Arr[k] = bezier_pnt(ipnt,ipnt+hr,aapnt+wr,aapnt,j*dtheta);
       #declare N_vec_Arr[k] = vnormalize(V_vec_Arr[k]-aapnt); // set something, actual=Norms()
       #if (j=1)
           #declare Edge_Arr[NumEdges] = <i,k>;
           #declare NumEdges = NumEdges+1;
        #else
           #declare Edge_Arr[NumEdges] = <k-1,k>;
           #declare NumEdges = NumEdges+1;           
       #end
       #if (j=slices-2)  // last one
           #declare Edge_Arr[NumEdges] = <k,aa>;
           #declare NumEdges = NumEdges+1;
       #end
 
      #local k=k+1; #declare NumVertices = k;
      #local j=j+1;
    #end

    #if (i) 
      #if (Swtch)
        #local Swtch = 0;
        // incremented backbone last time   aa               
        #local j=1;
        #while (j < slices-2)
           #if (j=1)
              setFace(<aa-incdec,k-j-(slices-2),k-j>); //      _|
              setFace(<aa-incdec,k-j,aa>);      //             |/
           #end
           setFace(<k-j-(slices-2),k-j-(slices-2)-1,k-j-1>);  //      _|
           setFace(<k-j-(slices-2),k-j-1,k-j>); //                    |/
           //#end                               )
           #local j=j+1; 
        #end
        //setFace(<k-j-(slices-2)-1,i,k-j-1>); //             \/
        setFace(<k-j-(slices-2),i,k-j>); //             \/
        
        //#if (i = bsize-1) /// hmm  #end assuming 1st and last point connect same bakbone pnt1 ?!? fuzzy logic
      #else 
        #local j=1;
        //setFace(<aa,k-j-(slices-2),k-j>);
        setFace(<aa-incdec,k-j-(slices-2),k-j>);         // _|
        setFace(<aa-incdec,k-j,aa>);                     // |/
        #while (j < slices-1)
           #if (j=slices-2)
              setFace(<k-j,k-j-(slices-2),i>);
              setFace(<k-j-(slices-2),i-1,i>);
           #else
              setFace(<k-j,k-j-(slices-2),k-j-1>);
              setFace(<k-j-(slices-2),k-j-(slices-2)-1,k-j-1>);
           #end
           #local j=j+1;
        #end      
      #end// swtch
      
      #if ((i = bsize-1) & (close)) //done. close circular shape
         #local j=1;
         setFace(<aa,k-j,kk-j>);
         
        //setFace(<aa-1,k-j,kk-j>);         // _|
        //setFace(<aa-1,k-j,kk-j,aa>);                     // |/
        
        
         #while (j < slices-1)
            #if (j=slices-2)
               setFace(<kk-j,k-j,0>);
               setFace(<k-j,i,0>);
            #else
               setFace(<kk-j,k-j,kk-j-1>);
               setFace(<k-j,k-j-1,kk-j-1>);
            #end
            
            #local j=j+1;
         #end
      #end
    #end // i
    #local i=i+1;      #if (0) // for pov logo only                                             // corner of 50
                          #if ((i>44) & (i<52)) #local trap = 0;   // fix for perimeter not following bakbone
                          #else #local trap = 1; #end
                       #end
                    //n
    #if (asize > 1)
    #if (aa >= n-1) #local incdec = -1; #end
    //#if ((aa+incdec > bsize-1) & (i < bsize))     // stay in range
         //#if ((vlength(V_vec_Arr[aa+incdec]-bSpline[i])<vlength(V_vec_Arr[aa]-bSpline[i])) & (trap))
 #local aa=aa+incdec;
             // if next backbone pnt is closer, use it
            // #local Swtch = 1;
            // #local i=i-1;  // dont goto next perimeter pnt
        // #end
    //#end
    #end
  #end
  
  echo("\nNum Verts ********* ")
  echoi(NumVertices)
  echo("\nNum Faces ********* ")
  echoi(NumFaces)
  echo("\nNum Edges ********* ")
  echoi(NumEdges) cr() cr()
#end